@@ -55,6 +55,8 @@ import pydal
 import pluralize
 from pydal._compat import to_native, to_bytes
 
+bottle.BaseRequest.MEMFILE_MAX = 16 * 1024 * 1024
+
 __all__ = [
     "render",
     "DAL",
@@ -392,9 +394,7 @@ class Session(Fixture):
             except Exception:
                 pass
         if not "uuid" in self.local.data:
-            self.local.changed = True
-            self.local.data["uuid"] = str(uuid.uuid1())
-            self.local.data["secure"] = self.local.secure
+            self.clear()
 
     def save(self):
         self.local.data["timestamp"] = time.time()
@@ -433,6 +433,8 @@ class Session(Fixture):
     def clear(self):
         self.local.changed = True
         self.local.data.clear()
+        self.local.data["uuid"] = str(uuid.uuid1())
+        self.local.data["secure"] = self.local.secure
 
     def on_request(self):
         self.load()
