@@ -1,5 +1,5 @@
 import uuid
-import hmac
+import hashlib, hmac
 from py4web import DAL, request
 from yatl.helpers import (
     A,
@@ -44,7 +44,7 @@ def FormStyleDefault(table, vars, errors, readonly, deletable, classes=None):
         if field.type == "id" and value is None:
             continue
         if readonly or field.type == "id":
-            control = DIV(field.represent and field.represent(value) or value or "")            
+            control = DIV(field.represent and field.represent(value) or value or "")
         elif field.widget:
             control = field.widget(table, value)
         elif field.type == "text":
@@ -175,6 +175,7 @@ class Form(object):
     - dbio: set to False to prevent any DB writes
     - keep_values: if set to true, it remembers the values of the previously submitted form
     - form_name: the optional name of this form
+    - csrf_session: if None, no csrf token is added.  If a session, then a CSRF token is added and verified.
     """
 
     def __init__(
@@ -189,6 +190,7 @@ class Form(object):
         form_name=False,
         hidden=None,
         validation=None,
+        csrf_session=None,
     ):
 
         if isinstance(table, list):
@@ -219,14 +221,10 @@ class Form(object):
         self.hidden = hidden
         self.formkey = None
         self.cached_helper = None
+        self.csrf_session = csrf_session
 
         if readonly or request.method == "GET":
-            if self.record:
-                self.vars = {
-                    name: table[name].formatter(self.record[name])
-                    for name in table.fields
-                    if name in self.record
-                    }
+            self._read_vars_from_record(table)
         else:
             post_vars = request.forms
             self.submitted = True
@@ -235,12 +233,12 @@ class Form(object):
             # We only a process a form if it is POST and the formkey matches (correct formname and crsf)
             # Notice: we never expose the crsf uuid, we only use to sign the form uuid
             if request.method == "POST":
-                if post_vars.get("_formkey") == self.form_name:
+                if self._verify_form(post_vars):
                     process = True
             if process:
                 if not post_vars.get("_delete"):
                     for field in self.table:
-                        if field.writable:
+                        if field.writable and field.readable:
                             value = post_vars.get(field.name)
                             record_id = self.record and self.record.get("id")
                             (value, error) = field.validate(value, record_id)
@@ -268,8 +266,51 @@ class Form(object):
                 elif dbio:
                     self.deleted = True
                     self.record.delete_record()
-        # Store key for future CSRF
-        self.formkey = self.form_name
+            else:
+                # This form should not be processed.  We return the same as for GET.
+                self._read_vars_from_record(table)
+        self._sign_form()
+
+    def _read_vars_from_record(self, table):
+        if self.record:
+            if isinstance(table, list):
+                # The table is just a list of fields.
+                self.vars = {field.name: self.record.get(field.name)
+                             for field in table}
+            else:
+                self.vars = {
+                    name: table[name].formatter(self.record[name])
+                    for name in table.fields
+                    if name in self.record
+                }
+
+    def _get_signature(self, salt=""):
+        key = self.csrf_session.get("_form_key")
+        if key is None:
+            key = str(uuid.uuid1())
+            self.csrf_session["_form_key"] = key
+        h = hmac.new(key.encode("utf8"), msg=salt.encode("utf8"), digestmod=hashlib.sha256)
+        return h.hexdigest()
+
+    def _sign_form(self):
+        """Signs the form, for csrf"""
+        # Adds a form key.  First get the signing key from the session.
+        if self.csrf_session is not None:
+            salt = str(uuid.uuid4())
+            self.formkey = salt + ";" + self._get_signature(salt)
+
+    def _verify_form(self, post_vars):
+        """Verifies the csrf signature and form name."""
+        if post_vars.get("_formname") != self.form_name:
+            return False
+        if not self.csrf_session:
+            return True
+        formkey = post_vars.get("_formkey")
+        try:
+            salt, u = formkey.split(";")
+            return u == self._get_signature(salt=salt)
+        except:
+            return False
 
     def update_or_insert(self):
         if self.record:
@@ -291,6 +332,10 @@ class Form(object):
             helper = self.formstyle(
                 self.table, self.vars, self.errors, self.readonly, self.deletable
             )
+            if self.form_name:
+                helper.append(
+                    INPUT(_type="hidden", _name="_formname", _value=self.form_name)
+                )
             if self.formkey:
                 helper.append(
                     INPUT(_type="hidden", _name="_formkey", _value=self.formkey)
